#!/usr/bin/env python

import sys, os, tempfile
import Image

def parse_options(argv):
	options = {}
	options["obj_path"] = argv.pop(0)
	options["img_width"] = int(argv.pop(0))
	options["img_height"] = int(argv.pop(0))
	options["size_factor"] = float(argv.pop(0))
	while argv:
		cmd = argv.pop(0)
		
		# Extra options for renderstrip
		if cmd == "--output" or cmd == "-o":
			options["output"] = argv.pop(0)
		elif cmd == "--num-images":
			options["num_images"] = int(argv.pop(0))
		elif cmd == "--pixellate":
			options["pixellate"] = int(argv.pop(0))
		elif cmd == "--columns":
			options["columns"] = int(argv.pop(0))
		elif cmd == "--yaw":
			options["yaw"] = int(argv.pop(0))
			
		# Options to pass to RetroRenderer
		elif cmd == "--pitch":
			options["pitch"] = int(argv.pop(0))
		elif cmd == "--light-angle":
			options["light_angle"] = [float(argv.pop(0)), float(argv.pop(0)), float(argv.pop(0))]
		elif cmd == "--light-color":
			options["light_color"] = [float(argv.pop(0)), float(argv.pop(0)), float(argv.pop(0))]
		elif cmd == "--cull":
			options["cull"] = argv.pop(0)
		elif cmd == "--autocompute-normals":
			options["autocompute_normals"] = True
		else:
			raise Exception("Don't understand argument \"%s\"" % cmd)
	return options

def runcmd(cmd, args):
	
	print "%s %s" % (cmd, " ".join(args))
	result = os.spawnv(os.P_WAIT, cmd, [cmd]+args)
	if result != 0:
		raise Exception("%s exited with return code %d" % (cmd,result))

def render(options):
	
	output_fn = tempfile.mkstemp(".tga", "RetroRender-")[1]
	
	args = []
	args.append(options["obj_path"])
	args.append(str(options["img_width"]))
	args.append(str(options["img_height"]))
	args.append(str(options["size_factor"]))
	args.extend(["--output", output_fn])
	if "pitch" in options:
		args.extend(["--pitch", "%d" % options["pitch"]])
	if "yaw" in options:
		args.extend(["--yaw", "%d" % options["yaw"]])
	if "light_angle" in options:
		args.append("--light-angle")
		args.extend("%5f"%x for x in options["light_angle"])
	if "light_color" in options:
		args.append("--light-color")
		args.extend("%5f"%x for x in options["light_color"])
	if "cull" in options:
		args.append("--cull")
		args.append(options["cull"])
	if "autocompute_normals" in options:
		args.append("--autocompute-normals")
	
	runcmd(os.path.join(os.path.split(sys.argv[0])[0], "RetroRenderer"), args)
	
	img = Image.open(output_fn)
	
	os.remove(output_fn)
	
	return img

opts = parse_options(sys.argv[1:])
num_images = opts["num_images"]
strip_output_fn = opts["output"]

if "columns" in opts:
	num_columns = opts["columns"]
	num_rows = ((num_images-1)/num_columns)+1
else:
	num_columns = num_images
	num_rows = 1

strip = Image.new("RGB", (opts["img_width"]*num_columns, opts["img_height"]*num_rows), (127,127,127))


for i in xrange(num_images):
	angle = i*360/num_images + opts.get("yaw", 0)
	opts2 = opts.copy()
	opts2["yaw"] = angle
	img = render(opts2)
	
	if "columns" in opts:
		x = i % opts["columns"]
		y = i / opts["columns"]
	else:
		x = i
		y = 0
	strip.paste(img, (opts["img_width"]*x, opts["img_height"]*y))

if "pixellate" in opts:
	factor = opts["pixellate"]
	strip = strip.resize((strip.size[0]*factor, strip.size[1]*factor), Image.NEAREST)

strip.save(strip_output_fn)	